-- 古镇秘藏 地图panel
checkRequire("Lua_TownTreasureCellItem");
checkRequire("Lua_TownTreasureTipItem");
Lua_TownTreasureMapPanel = class("Lua_TownTreasureMapPanel", Lua_DisplayObject);

function Lua_TownTreasureMapPanel:ctor(p)
    self:superCall("ctor", p);
end

function Lua_TownTreasureMapPanel:buildPanel()
    self:superCall("buildPanel");

    self:addLuaEvent("TOWN_TREASURE_INFO_UPDATE", self, self.updatePanel);
    self:addLuaEvent("TOWN_TREASURE_PLAY", self, self.onStartMove);
    self:addLuaEvent("TOWN_TREASURE_MAP_MOVE", self, self.moveToCell);
    self:addLuaEvent("TOWN_TREASURE_BUILDING_UPDATE", self, self.updateBuildCell);

    self:addLuaEvent("TOWN_TREASURE_ADD_ITEM_TIP", self, self.onAddItemTip);
    self:addLuaEvent("TOWN_TREASURE_PLAY_STOP", self, self.stopModelMove);
    self:addLuaEvent("TOWN_TREASURE_PLAY_DELAY", self, self.delayModelMove);
    self:addLuaEvent("TOWN_TREASURE_PLAY_TRANSFER", self, self.setTransfer);
    self:addLuaEvent("TOWN_TREASURE_PLAY_RUN", self, self.setRun);
    self:addLuaEvent("TOWN_TREASURE_GET_EFFECT_PLAY", self, self.showGetEff);
    self:addLuaEvent("TOWN_TREASURE_ITEM_FLY", self, self.showItemFly);
    self:addLuaEvent("TOWN_TREASURE_TIME_RESET", self, self.resetModelMove);
end

function Lua_TownTreasureMapPanel:buildShowItem()
    self:superCall("buildShowItem");

    self.curVersion = 0;
    self.cellItemList = {};
    self.isDraging = false;
    self.isCamFllow = true;
    self.isTransfer = false;
    self.isRun = false;
    self.runNum = 0;
    self.isOnMoving = false;

    self.moveSpeed = DataCenter.townTreasureUtil.moveSpeed;

    self.rectWidth = 720;
    self.rectHeight = 1440 + self.adaptHeight;

    self.scrollContainer = UICreate.createScrollContainer(self.content, 0, 0, self.rectWidth, self.rectHeight);
    self.scrollContainer.bCreateContent = true;
    self.scrollContainer.horizontal = true;
    self.scrollContainer.vertical = true;
    self.scrollContainer.movementTypeValue = 2;
    self.scrollContainer.elasticity = 0.3;
    self.scrollContainer.inertia = false;
    self.scrollContainer:setScrollPivot(0);
    self.scrollContainer:setScrollAnchor(0, 1, 0, 1);
    -- self.scrollContainer:addRectMask2D();
    self.scrollContainer.siblingIndex = 1;
    self.scrollContainer:addMousDownFuntion(self, self.onMouseDownFun);
    self.scrollContainer:addMouseUpFuntion(self, self.onMouseUpFun);

    self.sub = self.scrollContainer.scrollContent;
    self.subRect = self.scrollContainer.scrollRect;

    self.mapBg = UICreate.createImage(self.sub, "/images/townTreasure/xui_gcmz_bg_1.jpg", 0, 0, 0, 0);
    self.mapBg.anchorValue = 0;
    self.mapBg.pivotValue = 0;

    self.targetEff = UICreate.createUIEffect(self.sub, "/effects/uiEffects/ui_gcmb_dg_01", "ui_gcmb_dg_01", 0, 0);
    self.targetEff.anchorValue = 0;

    self.tweenObj = UICreate.createGameObject(self.sub, 0, 0, 0);
    self.tweenObj.name = "tweenObj";

    self.modelObj = UICreate.createGameObject(self.sub, 0, 0, 0);
    self.modelObj.name = "modelObj";

    self.modelShadow = UICreate.createUIAtlasImage(self.modelObj, "townTreasure_atlas", "xui_gcmz_window_24", 0, 15, 0, 0);

    self.modelEffImg = UICreate.createModelEffImg(self.modelObj, "/model/role/109000100_low", "109000100_low", 0, -60, 400, 400);
    self.modelEffImg:setCamField(1, 1, 100, true);
    self.modelEffImg:setCamPos(7.36, 8, -6.23);
    self.modelEffImg:setCamRot(30, -50, 0);
    self.modelEffImg:setCamField(3, 1, 100, true);
    self.modelEffImg.canRot = false;
    self.modelEffImg.anchorValue = 0;
    self.modelEffImg.pivotValue = 7;
    self.modelEffImg:setAction(0);
    self.modelEffImg:setModelEffDir(120);

    local scale = DataCenter.townTreasureUtil.modelScale;
    self.modelEffImg:setScale(scale, scale, 1);

    self.startEff = UICreate.createUIEffect(self.sub, "/effects/uiEffects/ui_gcmb_chushengdian_01", "ui_gcmb_chushengdian_01", 0, 0);
    self.startEff.anchorValue = 0;
    
    self:startSlowTween();

    if DataCenter.townTreasureData.cellDic ~= nil then
        self:updatePanel();
    end
end

function Lua_TownTreasureMapPanel:onScreenResize(adaptHeight)
    self:superCall("onScreenResize", adaptHeight);

    self.rectHeight = 1440 + self.adaptHeight;
    if self.scrollContainer ~= nil then
        self.scrollContainer:SetDimensions(self.rectWidth, self.rectHeight);
    end
end

function Lua_TownTreasureMapPanel:dispose()
    DataCenter.townTreasureData.isMoving = false;
    DataCenter.townTreasureData.isTheWorld = false;

    Lua_CommonUtil.removeTimer(self, self.toNextStep);
    Lua_CommonUtil.removeTimer(self, self.selectCellClick);
    TweenUtil.getInstance():stop(self.tweenObj);
    
    if self.scrollContainer ~= nil then
        self.scrollContainer:dispose();
        self.scrollContainer = nil;
    end
    if self.mapBg ~= nil then
        self.mapBg:dispose();
        self.mapBg = nil;
    end
    if self.modelShadow ~= nil then
        self.modelShadow:dispose();
        self.modelShadow = nil;
    end
    if self.modelEffImg ~= nil then
        self.modelEffImg:removeHangerEff();
        self.modelEffImg:dispose();
        self.modelEffImg = nil;
    end
    if self.cellItemList ~= nil then
        for k, v in pairs(self.cellItemList) do
            v:dispose();
            v = nil;
        end
        self.cellItemList = nil;
    end
    if self.tipShowArr ~= nil then
        self.tipShowArr:dispose();
        self.tipShowArr = nil;
    end
    if self.tipHideArr ~= nil then
        self.tipHideArr:dispose();
        self.tipHideArr = nil;
    end
    if self._getEff ~= nil then
        self._getEff:dispose();
        self._getEff = nil;
    end
    if self.targetEff ~= nil then
        self.targetEff:dispose();
        self.targetEff = nil;
    end
    if self.startEff ~= nil then
        self.startEff:dispose();
        self.startEff = nil;
    end
    if self.citieEff ~= nil then
        self.citieEff:dispose();
        self.citieEff = nil;
    end
    if self.modelObj ~= nil and IsNil(self.modelObj) == false then
        UnityEngine.GameObject.Destroy(self.modelObj);
        self.modelObj = nil;
    end
    if self.tweenObj ~= nil and IsNil(self.tweenObj) == false then
        UnityEngine.GameObject.Destroy(self.tweenObj);
        self.tweenObj = nil;
    end
	self:superCall("dispose");
end

function Lua_TownTreasureMapPanel:updatePanel()
    local data = DataCenter.townTreasureData;
    if self.curVersion ~= data.version then
        self.curVersion = data.version;
        self.subRectW, self.subRectH = DataCenter.townTreasureUtil:getMapSize(self.curVersion);
        self.scrollContainer:setScrollSize(self.subRectW, self.subRectH);

        local mapPath = DataCenter.townTreasureUtil:getMapBg(self.curVersion);
        local mapPos = DataCenter.townTreasureUtil:getVersionInfo(self.curVersion, "map_offset_pos");
        self.mapBg.resetBgSize = true;
        self.mapBg:setUrl(mapPath);
        self.mapBg:setLocation(mapPos[1], mapPos[2]);

        local cellVo = data.cellDic[data.roadArr[1]];
        self.startEff:setLocation(cellVo.posX, -cellVo.posY, 0);

        for k, v in pairs(self.cellItemList) do
            v:dispose();
            v = nil;
        end
    end

    if data.isTheWorld then
        return;
    end

    local item;
    for k, v in pairs(data.cellDic) do
        item = self.cellItemList[k];
        if item == nil then
            item = Lua_TownTreasureCellItem.new(self.sub, self.tweenObj);
            item:setSiblingIndex(1);
            self.cellItemList[k] = item;
        end
        item:setData(v);
    end

    local cellData = data.cellDic[data.curCellId]
    self:setModelPos(cellData.posX, -cellData.posY);
    self:cheeckModelAngle();
    self:setTargetEff(true, cellData.posX, -cellData.posY);

    self.startEff:setEnable(data.curCellId ~= data.roadArr[1]);

    if not self.isFirstIn then
        self.isFirstIn = true;
        self:setCamMove(cellData.posX, -cellData.posY, false, false);
    end

    if self.shouRunEff then
        self.shouRunEff = false;
        self.modelEffImg:removeHangerEff("chongfeng_01");
    end

    if self.shouCitiEff then
        self.shouCitiEff = false;
        -- self.modelEffImg:removeHangerEff("gcmb_citie_01");
        if self.citieEff ~= nil then
            self.citieEff:setEnable(false);
        end
    end
end

function Lua_TownTreasureMapPanel:updateBuildCell(idArr)
    local item;
    for i = 1, idArr:getSize() do
        item = self.cellItemList[idArr[i]];
        item:updateData();
    end
end

function Lua_TownTreasureMapPanel:onMouseDownFun()
    self:removeTimer(self, self.onCamMove);
    self.isCamMove = false;
    self.isCamFllow = false;
    self.isDraging = true;
    
    self.startMouseX = CommonUtil.mousePositionX;
    self.startMouseY = CommonUtil.mousePositionY;
    self.startClickTime = getTime();
    self.checkItem = self:checkPressItem();
end

function Lua_TownTreasureMapPanel:onMouseUpFun()
    self.isDraging = false;
    if self.checkItem == nil then return end

    local dx = CommonUtil.mousePositionX - self.startMouseX;
    local dy = CommonUtil.mousePositionY - self.startMouseY;
    if getTime() - self.startClickTime <= 0.3 and Mathf.Abs(dx) + Mathf.Abs(dy) < 4 then
        self.checkItem:onCellClick();
    end
end

function Lua_TownTreasureMapPanel:checkPressItem()
    if self.cellItemList ~= nil then
        local winPos = LuaUtil.getTouchPosInverseObj(self.sub);
        for k, v in pairs(self.cellItemList) do
            if v.isActive and v:checkInBound(winPos.x, winPos.y) then
                return v;
            end
        end
    end
    return nil;
end

function Lua_TownTreasureMapPanel:setModelPos(posX, posY)
    self.modelPosX = posX;
    self.modelPosY = posY;
    -- self.modelEffImg:setLocation(posX, posY - 20);
    LuaUtil.setObjPos(self.modelObj, posX, posY - 20, 0);
    -- self:setCamMove(posX, posY, false, false);
end

function Lua_TownTreasureMapPanel:onStartMove()
    -- self:setCamMove(self.modelPosX, self.modelPosY, false, true);

    local data = DataCenter.townTreasureData;
    if data.playCellInfo[data.curCellId] ~= nil then -- 离开刷新格子
        local cellData = data.cellDic[data.curCellId];
        data:refreshCellData(cellData.id);
        self.cellItemList[cellData.id]:updateData();
    end

    if self.isTransfer then
        self.isTransfer = false;
        data.curCellId = data.targetCellId;
        if data.playCellInfo[data.targetCellId] ~= nil then
            local cellData = data.cellDic[data.targetCellId];
            local cellId = cellData.id;
            if cellData.bind_build_id ~= nil then
                cellId = cellData.bind_build_id;
            end
            self.cellItemList[cellId]:onShowTip();
            data:refreshCellData(cellData.id);
            self.cellItemList[cellId]:updateData();
            self:setModelPos(cellData.posX, -cellData.posY);
        end
        self:stopModelMove();
        self:toNextStep();

        if data.roadArr:indexOf(data.curCellId) < self.targetIdx then -- 超过起点
            Lua_ProxyManager.getProxy("Lua_TownTreasureProxy"):sActivityInfo();
        end
        return;
    end

    if self.isRun and self.runNum >= 1 then
        self.isRun = false;
        self.runNum = 0;
    end

    if self.isRun then
        self.runNum = self.runNum + 1;
        self.modelEffImg:setAction(8);
        self.modelEffImg:addHangerEff("chongfeng_01");
        self.shouRunEff = true;
    else
        self.modelEffImg:setAction(101);
    end

    self.curIdx = data.roadArr:indexOf(data.curCellId);
    self.targetIdx = data.roadArr:indexOf(data.targetCellId);
    -- OperateAlert.show(LanUtil.formateStr("开始走{1}步", data.diceNum));

    local targetCellVo = data.cellDic[data.targetCellId];
    self:setTargetEff(true, targetCellVo.posX, -targetCellVo.posY);

    if not self.isOnMoving then
        self.isOnMoving = true;
        self.cutTime = getTime();
        -- DataCenter.loginData:checkFpsState();
        self:addTimer("onModelMoving", 0.02, self, self.onModelMoving);
    end
end

function Lua_TownTreasureMapPanel:onModelMoving()
    local data = DataCenter.townTreasureData;
    if self.curIdx == self.targetIdx then
         if self.isResetData and data.diceInfoArr:getSize() <= 0 then
            self.isResetData = false;
            Lua_ProxyManager.getProxy("Lua_TownTreasureProxy"):sActivityInfo();
        end

        self:stopModelMove();
        self:toNextStep();
        -- DataCenter.loginData:checkFpsState();
        return;
    end

    local idx;
    if data.diceNum >= 0 then
        idx = self.curIdx + 1;
        if idx > data.roadArr:getSize() then
            idx = 1;
        end
    else
        idx = self.curIdx - 1;
        if idx < 1 then
            idx = data.roadArr:getSize();
        end
    end

    local dt = getTime() - self.cutTime;
    self.cutTime = getTime();
    local speed = self.isRun and self.moveSpeed[2] or self.moveSpeed[1];
    local moveLen = speed * dt;
    
    local cellData = data.cellDic[data.roadArr[idx]];
    local disX = cellData.posX - self.modelPosX;
    local disY = -cellData.posY - self.modelPosY;
    local distance = Mathf.Sqrt(disX * disX + disY * disY);
    self:setCamMove(cellData.posX, -cellData.posY, false, false);
    if moveLen >= distance or distance <= 2 then
        local cellId = cellData.id;
        data.curCellId = cellId;
        if data.playCellInfo[cellId] ~= nil then
            if cellData.bind_build_id ~= nil then
                cellId = cellData.bind_build_id;
            end
            self.cellItemList[cellId]:onShowTip();
            data:refreshCellData(cellData.id);
            self.cellItemList[cellId]:updateData();
        end

        self.curIdx = idx;
        self:setModelPos(cellData.posX, -cellData.posY);
        if data.diceNum > 0 and self.curIdx == 1 and DataCenter.townTreasureData.loopTime > 0 then
            self.isResetData = true; -- 经过终点重置
            self:stopModelMove();
            Lua_WindowUtil.openWin("Lua_TownTreasureLoopRewardWindow", Lua_Window.ACTION_OPEN);

            if self.curIdx ~= self.targetIdx then
                self.runNum = 0;
            end
        end

        self.startEff:setEnable(cellId ~= data.roadArr[1]);
    else
        local rate = moveLen / distance;
        self:setModelPos(self.modelPosX + disX * rate, self.modelPosY + disY * rate);
    end

    local offset = self.isRun and 45 or 20;
    local angle = Mathf.Atan2(-disY, disX) * 180 / Mathf.PI;
    self.modelEffImg:setModelEffDir(angle + offset);
end

function Lua_TownTreasureMapPanel:stopModelMove()
    self:removeTimer(self, self.onModelMoving);
    self.isOnMoving = false;

    self.modelEffImg:setAction(0);
    self:cheeckModelAngle();
end

function Lua_TownTreasureMapPanel:resetModelMove()
   self:stopModelMove();
   self:setCamMove(self.modelPosX, self.modelPosY, false, false);
end

function Lua_TownTreasureMapPanel:cheeckModelAngle()
    local townTreasureData = DataCenter.townTreasureData;
    local curCellVo = townTreasureData.cellDic[townTreasureData.curCellId];
    local nextCellVo = townTreasureData.cellDic[curCellVo.next_stage[1]];
    local disX = nextCellVo.posX - curCellVo.posX;
    local disY = nextCellVo.posY - curCellVo.posY;
    local angle = Mathf.Atan2(disY, disX) * 180 / Mathf.PI + 20;
    self.modelEffImg:setModelEffDir(angle);
end

function Lua_TownTreasureMapPanel:delayModelMove(delayTime)
    self:stopModelMove();
    Lua_CommonUtil.delayCall(delayTime, self, self.toNextStep);
end

function Lua_TownTreasureMapPanel:toNextStep()
    DataCenter.townTreasureData:setDiceInfo();
end

function Lua_TownTreasureMapPanel:setTransfer(val)
    self.isTransfer = val;
end

function Lua_TownTreasureMapPanel:setRun(val)
    self.isRun = val;
    self.runNum = 0;
end

function Lua_TownTreasureMapPanel:moveToCell(id)
    local data = DataCenter.townTreasureData.cellDic[id];
    local offset = data.build_offset_pos;
    self:setCamMove(data.posX + offset[1], data.posY + offset[2]);

    Lua_CommonUtil.delayCall(1.5, self, self.selectCellClick, id);
end

function Lua_TownTreasureMapPanel:selectCellClick(id)
    local item = self.cellItemList[id];
    if item ~= nil then
        item:onCellClick();
    end
end

function Lua_TownTreasureMapPanel:setCamMove(posX, posY, isNow, isLimit)
    if (DataCenter.townTreasureData.isMoving and self.isCamFllow == false) or self.isDraging then return end;

    local camPosX = -(posX - self.rectWidth / 2);
    if camPosX > 0 then
        camPosX = 0;
    elseif camPosX < -(self.subRectW - self.rectWidth) then
        camPosX = -(self.subRectW - self.rectWidth);
    end
    local camPosY = -(posY + self.rectHeight / 2);
    if camPosY > self.subRectH - self.rectHeight then
        camPosY = self.subRectH - self.rectHeight;
    elseif camPosY < 0 then
        camPosY = 0;
    end

    if isLimit then -- 避免镜头频繁跟随，超过范围才调整
        local posX = self.subRect.anchoredPosition.x;
        local posY = self.subRect.anchoredPosition.y;
        if Mathf.Abs(camPosX - posX) < self.rectWidth / 3 and Mathf.Abs(camPosY - posY) < self.rectHeight / 3 then
            return;
        else
            local sign = camPosX - posX > 0 and 1 or -1;
            camPosX = camPosX + sign * (self.rectWidth / 3);
        end
    end

    self.camPosX = camPosX;
    self.camPosY = camPosY;

    if isNow then
        self.scrollContainer:setScrollPosition(self.camPosX, self.camPosY, 0);
    end

    if not self.isCamMove then
        self.isCamMove = true;
        DataCenter.loginData:checkFpsState();
        self:addTimer("onCamMove", 0.02, self, self.onCamMove);
    end
end

function Lua_TownTreasureMapPanel:onCamMove()
    local posX = self.subRect.anchoredPosition.x;
    local posY = self.subRect.anchoredPosition.y;
    local disX = self.camPosX - posX;
    local disY = self.camPosY - posY;
    local distance = Mathf.Sqrt(disX * disX + disY * disY);
    if (DataCenter.townTreasureData.isMoving and self.isCamFllow == false) or self.isDraging or distance <= 5 then
        self:removeTimer(self, self.onCamMove);
        DataCenter.loginData:checkFpsState();
        self.isCamMove = false;
        self.scrollContainer:setScrollPosition(self.camPosX, self.camPosY, 0);
    else
        if Mathf.Abs(disX * 0.1) >= 2 then
            self.scrollContainer:setScrollPosition(posX + disX * 0.1, posY + disY * 0.1, 0);
        end
    end
end

function Lua_TownTreasureMapPanel:onAddItemTip(data)
    local item = self:getTipItem();
    item:setLocation(data.posX, data.posY + 120, 0);
    item:setData(data);

    TweenUtil.getInstance():stop(item.content, true);
    item:setAlphaVal(1);

    TweenUtil.getInstance():addTfTween(item.content.transform, 1, 0.5,
        data.posX, data.posY + 200, 0,
        0, 0, 0,
        1, 1, 1, self, self.returnTipItem, 0, false, 0);
    TweenUtil.getInstance():addTweenTargetAlpha(item.content, 2.5, 0.5, -1, false, nil, nil);
end

function Lua_TownTreasureMapPanel:getTipItem()
    if self.tipShowArr == nil then
        self.tipShowArr = Array();
        self.tipHideArr = Array();
    end

    local item = self.tipHideArr:shift();
    if item == nil then
        item = Lua_TownTreasureTipItem.new(self.sub);
        item:setRectAnchor(0);
    else
        TweenUtil.getInstance():stop(item.content, true);
        item:setAlphaVal(1);
        item:setEnable(true);
    end
    self.tipShowArr:add(item);
    return item;
end

function Lua_TownTreasureMapPanel:returnTipItem()
    if self.tipShowArr:getSize() > 0 then
        local item = self.tipShowArr:shift();
        TweenUtil.getInstance():stop(item.content, true);
        item:setAlphaVal(1);
        item:setEnable(false);
        self.tipHideArr:add(item);
    end
end

function Lua_TownTreasureMapPanel:showGetEff()
    if self._getEff == nil then
        self._getEff = UICreate.createUIEffect(self.modelObj, "/effects/uiEffects/ui_gcmb_lingqu_01", "ui_gcmb_lingqu_01", 0, 50);
    end
    self._getEff:setEnable(false);
    self._getEff:setEnable(true);
end

function Lua_TownTreasureMapPanel:startSlowTween()
    self.startTime = getTime();
    self:addTimer("onSlowTween", 0.02, self, self.onSlowTween);
end

function Lua_TownTreasureMapPanel:onSlowTween()
    local pt = (getTime() - self.startTime) * 5;
    LuaUtil.setObjPos(self.tweenObj, 0, 3 * math.sin(pt), 0);
end

-- function Lua_TownTreasureMapPanel:startSlowTween()
--     TweenUtil.getInstance():stop(self.tweenObj);
--     TweenUtil.getInstance():addTweenTargetPos(self.tweenObj, 2, 0,
--         0, 3, 0,
--         self, self.onSlowTween);
-- end


-- function Lua_TownTreasureMapPanel:startSlowTween()
--     self.isArraw = true;
--     self:addTimer("onSlowTween", 1.5, self, self.onSlowTween);
-- end

-- function Lua_TownTreasureMapPanel:onSlowTween()
--     TweenUtil.getInstance():stop(self.tweenObj);
--     local posY = self.isArraw and 5 or 0;
--     self.isArraw = not self.isArraw;
--     TweenUtil.getInstance():addTweenTargetPos(self.tweenObj, 2, 0,
--         0, posY, 0,
--         nil, nil);
-- end

function Lua_TownTreasureMapPanel:setTargetEff(val, posX, posY)
    -- if self._isSetTarget == val then return end
    -- self._isSetTarget = val;

    if val then
        self.targetEff:setLocation(posX, posY, 0);
        self.targetEff:setEnable(true);
    else
        self.targetEff:setEnable(false);
    end
end

function Lua_TownTreasureMapPanel:showItemFly()
    DataCenter.townTreasureData.isTheWorld = true;
    self.shouCitiEff = true;
    -- self.modelEffImg:addHangerEff("gcmb_citie_01");
    if self.citieEff == nil then
        self.citieEff = UICreate.createUIEffect(self.modelObj, "/effects/uiEffects/ui_gcmb_citie_01", "ui_gcmb_citie_01", 0, 120);
    else
        self.citieEff:setEnable(true);
    end

    local time = 1;
    for k, v in pairs(self.cellItemList) do
        if not v.data.is_building and v.data.eventId ~= nil and v.data.eventId ~= 0 and v.data.reward_type ~= 3 then
            v:onItemFly(self.modelPosX, self.modelPosY, time);
            time = time + 0.1;
        end
    end
end