-- 古镇秘藏 window
checkRequire("Lua_TownTreasureMapPanel");
checkRequire("Lua_TownTreasureBuffItem");
Lua_TownTreasureWindow = class("Lua_TownTreasureWindow", Lua_Window);

function Lua_TownTreasureWindow:ctor()
	self:superCall("ctor");

	self.isFullSizeWin = true;
	self.showBlackMask = true;
	self.showBackBtn = true;
	self.isBoxShow = true;

	self.windowName = "Lua_TownTreasureWindow";
	self.skinName = "TownTreasureWindow_skin";

	local mapPath = DataCenter.townTreasureUtil:getMapBg(DataCenter.townTreasureData.curVersion);
	if mapPath ~= nil then
		self:addPreLoadRes(mapPath, Lua_LoadFileType.IMG);
	end
end

function Lua_TownTreasureWindow:buildPanel(go)
	self:superCall("buildPanel", go);

	self:addLuaEvent("TOWN_TREASURE_INFO_UPDATE", self, self.updateInfo);
	self:addLuaEvent("TOWN_TREASURE_DICE_INFO", self, self.updateDiceInfo);
	self:addLuaEvent("TOWN_TREASURE_TASK_INFO", self, self.updateTaskInfo);

	self:addLuaEvent("TOWN_TREASURE_DICE_ANI", self, self.playDice);
	self:addLuaEvent("TOWN_TREASURE_PLAY_END", self, self.playEnd);
	self:addLuaEvent("TOWN_TREASURE_TIME_RESET", self, self.resetShow);
	self:addLuaEvent("TOWN_TREASURE_BUFF_CLICK", self, self.onBuffClick);
	self:addLuaEvent("TOWN_TREASURE_RESET_CHECK", self, self.resetCheckSelect);
end

function Lua_TownTreasureWindow:buildShowItem()
	self:superCall("buildShowItem");

	local diceItemVo = Lua_ToolVo.createToolVo(DataCenter.townTreasureUtil.commonDiceId);
	self.diceIcon:setAltas(diceItemVo.atlas, diceItemVo.sprite);
	self.diceIcon_1:setAltas(diceItemVo.atlas, diceItemVo.sprite);

	self.isShowBuff = DataCenter.townTreasureUtil:getConstConfig("show_buff_list", 1) == 1;
	self.buffItemArr = Array();

	local list = DataCenter.townTreasureUtil.getMoneyShow;
	local item;
	self.moneytItemArr = Array();
	for i = 1, #list do
		item = Lua_MoneyItem.new(self.content);
		item:setRectAnchor(1);
		item:setLocation(249 - (i - 1) * 190, -26, 0);
		item:setCanClick(true);
		item:setMoney(list[i]);
		self.moneytItemArr:add(item);
	end

	self.mapPanel = Lua_TownTreasureMapPanel.new(self.content);
	self.mapPanel:setRectAnchor(9);
	self.mapPanel:setSiblingIndex(0);

	self.taskItem = Lua_ItemIcon.new(self.content);
	self.taskItem:setScale(0.6, 0.6);
	self.taskItem:setRectAnchor(7);
	self.taskItem:setLocation(288, 407, 0);
	self.taskItem:setClick(self, self.onRewardIconClick);

	self.isClickAuto = false;
	self.checkBox.selected = DataCenter.townTreasureData.isOpenAuto;
	self.checkBox:addBoxChange(self, self.onBoxChange);

	self.tipBtn:addClickFuntion(self, self.onTipBtnClick);
	self.addBtn:addClickFuntion(self, self.onAddBtnClick);
	self.startBtn:addClickFuntion(self, self.onStartBtnClick);
	self.luckyIcon:addClickFuntion(self, self.onLuckyIconClick);
	self.viewIcon:addClickFuntion(self, self.onViewIconClick);
	self.buildImg:addClickFuntion(self, self.onViewIconClick);
	self.buildBg:addClickFuntion(self, self.onViewIconClick);

	self.buildImg.clickScale = false;
	self.buildBg.clickScale = false;

	DataCenter.redPointData:updateRedPointState(Lua_RedPointList.TownTreasure_Login, false, true);
	
	if DataCenter.townTreasureData.cellDic ~= nil then
		DataCenter.townTreasureData:checkLastEnd();
		self:updateInfo();
	else
		Lua_ProxyManager.getProxy("Lua_TownTreasureProxy"):sActivityInfo();
	end
end

function Lua_TownTreasureWindow:dispose()
	Lua_CommonUtil.removeTimer(self, self.playDiceAniEnd);

	if self.moneytItemArr ~= nil then
		self.moneytItemArr:dispose();
		self.moneytItemArr = nil;
	end
	if self.mapPanel ~= nil then
		self.mapPanel:dispose();
		self.mapPanel = nil;
	end
	if self.buffItemArr ~= nil then
		self.buffItemArr:dispose();
		self.buffItemArr = nil;
	end
	if self.taskItem ~= nil then
		self.taskItem:dispose();
		self.taskItem = nil;
	end
	if self.taskRedPointIcon ~= nil then
		self.taskRedPointIcon:dispose();
		self.taskRedPointIcon = nil;
	end
	if self.diceEff ~= nil then
		self.diceEff:dispose();
		self.diceEff = nil;
	end
	if self.taskEff ~= nil then
		self.taskEff:dispose();
		self.taskEff = nil;
	end
	self:superCall("dispose");
end

function Lua_TownTreasureWindow:updateInfo()
	local data = DataCenter.townTreasureData;

	local buildInfo = data.nearCellVo;
	if buildInfo.level == 0 then
		self.tipTxt:setLangValue(710032);
	else
		local itemVo = Lua_ToolVo.createToolVo(buildInfo.itemId);
		-- self.buildIcon:setAltas(buildInfo.addIcon[1], buildInfo.addIcon[2]);
		-- self.buildIcon:resetSize();
		self.buildImg:SetDimensions(0, 0);
		self.buildImg:setUrl(nil);
		self.buildImg:setUrl(buildInfo.img_path);
		self.buildImg:setScale(0.5, 0.5);
		self.buildItemIcon:setAltas(itemVo.iconAtlas, itemVo.iconSprite);
		-- self.buildItemIcon:SetDimensions(itemVo.iconWidth, itemVo.iconHeight);
		self.tipTxt.text = LanUtil.formateStr(710000, buildInfo.output_time - buildInfo.curTime, itemVo.name);
	end
	self.tipTxt:resetSize();
	if self.tipTxt.preferredHeight > 43 then
		self.buildBg:SetDimensions(155, 50 + (self.tipTxt.preferredHeight - 43));
	else
		self.buildBg:SetDimensions(155, 50);
	end

	self.buildImg:setEnable(buildInfo.level ~= 0);
	self.buildItemBg:setEnable(buildInfo.level ~= 0);
	self.buildItemIcon:setEnable(buildInfo.level ~= 0);
	self.viewIcon:setEnable(buildInfo.level ~= 0);

	self.time = data.closeTick - DataCenter.roleData.serverTime;
	self:starTimer();

	self:updateDiceInfo();
	self:updateBuff();
	self:updateLuckyDice();
end

function Lua_TownTreasureWindow:updateLuckyDice()
	local itemId = DataCenter.townTreasureUtil:getConstConfig("special_dice", 1);
	self.luckyTxt.text = DataCenter.packData:getItemCountById(itemId);
end

function Lua_TownTreasureWindow:updateDiceInfo()
	local data = DataCenter.townTreasureData;
	local maxNum = DataCenter.townTreasureUtil:getConstConfig("common_times_limit", 1);
	if data.diceTime <= 0 then
		self.isClickAuto = false;
		self:setAutoShow();
		self.numTxt.text = LanUtil.formateStr(710001, data.diceTime, maxNum);
	else
		self.numTxt.text = LanUtil.formateStr(710002, data.diceTime, maxNum);
	end

	self:updateTaskInfo();
end

function Lua_TownTreasureWindow:updateTaskInfo()
	local data = DataCenter.townTreasureData;
	if data.rewardState then
		if self.taskEff == nil then
			self.taskEff =  UICreate.createUIEffect(self.content, "/effects/uiEffects/ui_ty_daojulingqu", "ui_ty_daojulingqu", 288, 407);
			self.taskEff.anchorValue = 7;

			self.taskRedPointIcon = Lua_RedPointIcon.new(self.content, Lua_RedPointList.TownTreasure_Task);
			self.taskRedPointIcon:setRectAnchor(7);
			self.taskRedPointIcon:setLocation(325, 445, 0);
		end
		self.taskEff:setEnable(true);
		self.rewardTxt:setLangValue(710034);
	else
		if self.taskEff ~= nil then
			self.taskEff:setEnable(false);
		end
		local nextTaskInfo = DataCenter.townTreasureData:getNextTask(data.totalTime);
		local remainTime = nextTaskInfo.times - data.totalTime;
		if remainTime > 0 then
			self.rewardTxt.text = LanUtil.formateStr(710003, remainTime);
		else
			self.rewardTxt:setLangValue(710004);
		end
	end
	self.taskItem:setData(data.getTaskRewardShow);
	self:updateLuckyDice();
end

function Lua_TownTreasureWindow:updateBuff()
	if not self.isShowBuff then return end
	
	local arr = DataCenter.townTreasureData.buffArr;
	local count = arr:getSize();
	local len = self.buffItemArr:getSize();
	local num = count > len and count or len;
	local item;
	for i = 1, num do
		item = self.buffItemArr[i];
		if item == nil then
			item = Lua_TownTreasureBuffItem.new(self.content);
			item:setRectAnchor(1);
			self.buffItemArr:add(item);
		end

		if i <= count then
			item:setData(arr[i]);
			item:setLocation(-330 + (i - 1) * 40, -140 + (Mathf.Ceil(i / 4) - 1) * 60, 1);
		end
		item:setEnable(i <= count);
	end
end

function Lua_TownTreasureWindow:onBuffClick(data)
	local posX = -330 + (data.index - 1) * 40 + 130;
	local posY = -140 + (Mathf.Ceil(data.index / 4) - 1) * 60 - 65;
	local tipsData = { des = data.describe, posX = posX, posY = posY, maxWidth = 323, maxHeight = 90 };
	Lua_WindowUtil.openWin("Lua_InstructionTipsWindow", Lua_WinParam.ACTION_OPEN, tipsData);
end

function Lua_TownTreasureWindow:playDice()
	self:updateInfo();
	self.mapPanel.isCamFllow = true;

	-- 骰子动画
	if self.diceEff == nil then
		self.diceEff =  UICreate.createUIEffect(self.content, "/effects/uiEffects/ui_gcmb_touzi_01", "ui_gcmb_touzi_01", 0, 500);
		self.diceEff.anchorValue = 7;
	end
	self:playDiceAni()
end

function Lua_TownTreasureWindow:playDiceAni()
	self.diceEff:setEnable(true);
	self.diceEff:setSpineAniState(0, "0"..DataCenter.townTreasureData.diceNum);
	Lua_CommonUtil.delayCall(1, self, self.playDiceAniEnd);
	self.isPlayDice = true;
end

function Lua_TownTreasureWindow:playDiceAniEnd()
	self.isPlayDice = false;
	self.diceEff:setEnable(false);
	DataCenter.townTreasureData:setDiceInfo();
end

function Lua_TownTreasureWindow:playEnd()
	if DataCenter.townTreasureData.isTheWorld == false and DataCenter.townTreasureData.isOpenAuto and self.isClickAuto and DataCenter.townTreasureData.diceTime > 0 then
		Lua_ProxyManager.getProxy("Lua_TownTreasureProxy"):sExploreInfo();
	end
end

function Lua_TownTreasureWindow:starTimer()
	if not self.isCountDown then
		self.isCountDown = true;
		self:onCountDown();
		self:addTimer("onCountDown", 1, self, self.onCountDown);
	end 
end

function Lua_TownTreasureWindow:onCountDown()
	self.time = DataCenter.townTreasureData.closeTick - DataCenter.roleData.serverTime;
	local str = Mathf.Floor(self.time / 86400);
	if str >= 1 then
		str = Mathf.Ceil(self.time / 86400);
		self.timeTxt.text = LanUtil.formateStr(710005, str);
	else
		str = Mathf.Floor(self.time / 3600);
		if str > 1 then
			self.timeTxt.text = LanUtil.formateStr(710006, str);
		else
			str = Mathf.Floor(self.time / 60);
			if str < 0 then str = 0 end
			self.timeTxt.text = LanUtil.formateStr(710007, str);
		end
	end
end

function Lua_TownTreasureWindow:onTipBtnClick()
	if self.isPlayDice then
		OperateAlert.showLangValue(710033);
		return;
	end
	if DataCenter.townTreasureData.isMoving or DataCenter.townTreasureData.isTheWorld then
		OperateAlert.showLangValue(710008);
		return;
	end

	local tipsData = { des = LanUtil.getValue(710009), posX = 165, posY = -260 };
	Lua_WindowUtil.openWin("Lua_InstructionTipsWindow", Lua_WinParam.ACTION_OPEN, tipsData);
end

function Lua_TownTreasureWindow:onBoxChange(val)
	local limit = DataCenter.townTreasureUtil:getConstConfig("auto_throw_limit", 1);
	if DataCenter.townTreasureData.loopTime < limit then
		OperateAlert.show(LanUtil.formateStr(710010, limit));
		self.checkBox.selected = false;
		DataCenter.townTreasureData.isOpenAuto = false;
		return;
	end
	DataCenter.townTreasureData.isOpenAuto = val;
	
	if not self.isClickAuto and val then
		self.isBoxShow = false;
		self:onStartBtnClick();
	elseif not val then
		self.isClickAuto = false;
		self:setAutoShow();
	end
	self.isBoxShow = true;
end

function Lua_TownTreasureWindow:setAutoShow()
	if self.isClickAuto then
		self.startBtn:setLangValue(710012);
	else
		self.startBtn:setLangValue(710013);
	end
	self.autoTxtSp:setEnable(self.isClickAuto);
end

function Lua_TownTreasureWindow:onAddBtnClick()
	if self.isPlayDice then
		OperateAlert.showLangValue(710033);
		return;
	end
	if DataCenter.townTreasureData.isMoving or DataCenter.townTreasureData.isTheWorld then
		OperateAlert.showLangValue(710008);
		return;
	end
	Lua_WindowUtil.openWin("Lua_TownTreasureDiceBuyWindow", Lua_WinParam.ACTION_OPEN, true);
end

function Lua_TownTreasureWindow:onStartBtnClick()
	if self.clickTime and getTime() - self.clickTime <= 0.2 then
		OperateAlert.showLangValue(710011);
		return;
	end
	self.clickTime = getTime();

	local data = DataCenter.townTreasureData;
	if data.isOpenAuto then
		if self.isClickAuto == false and data.diceTime <= 0 then
			Lua_WindowUtil.openWin("Lua_TownTreasureDiceBuyWindow", Lua_WinParam.ACTION_OPEN, self.isBoxShow);
			return;
		end
		
		self.isClickAuto = not self.isClickAuto;
		self:setAutoShow();

		if self.isClickAuto == false then -- 取消状态不触发其它
			return;
		end
	end

	if data.isMoving or data.isTheWorld or self.isPlayDice then
		return;
	end

	if data.diceTime > 0 then
		Lua_ProxyManager.getProxy("Lua_TownTreasureProxy"):sExploreInfo();
	else
		Lua_WindowUtil.openWin("Lua_TownTreasureDiceBuyWindow", Lua_WinParam.ACTION_OPEN, true);
	end
end

function Lua_TownTreasureWindow:onRewardIconClick()
	if self.isPlayDice then
		OperateAlert.showLangValue(710033);
		return;
	end
	if DataCenter.townTreasureData.isMoving or DataCenter.townTreasureData.isTheWorld then
		OperateAlert.showLangValue(710008);
		return;
	end
	Lua_WindowUtil.openWin("Lua_TownTreasureTaskWindow", Lua_WinParam.ACTION_OPEN);
end

function Lua_TownTreasureWindow:onLuckyIconClick()
	if self.isPlayDice then
		OperateAlert.showLangValue(710033);
		return;
	end
	if DataCenter.townTreasureData.isMoving or DataCenter.townTreasureData.isTheWorld then
		OperateAlert.showLangValue(710008);
		return;
	end
	Lua_WindowUtil.openWin("Lua_TownTreasureDiceWindow", Lua_WinParam.ACTION_OPEN);
end

function Lua_TownTreasureWindow:onViewIconClick()
	if self.isPlayDice then
		OperateAlert.showLangValue(710033);
		return;
	end
	if DataCenter.townTreasureData.isMoving or DataCenter.townTreasureData.isTheWorld then
		OperateAlert.showLangValue(710008);
		return;
	end
	Lua_WindowUtil.openWin("Lua_TownTreasureViewWindow", Lua_WinParam.ACTION_OPEN);
end

function Lua_TownTreasureWindow:onBackBtnClick(go)
	if self.isPlayDice then
		OperateAlert.showLangValue(710033);
		self:resetCheckSelect();
		return;
	end
	if DataCenter.townTreasureData.isMoving or DataCenter.townTreasureData.isTheWorld then
		OperateAlert.showLangValue(710008);
		self:resetCheckSelect();
		return;
	end
	self:superCall("onBackBtnClick", go);
end

function Lua_TownTreasureWindow:resetShow()
	DataCenter.townTreasureData.isOpenAuto = false;
	DataCenter.townTreasureData.isTheWorld = false;
	DataCenter.townTreasureData.isMoving = false;
	self.isPlayDice = false;

	self.checkBox.selected = false;
	self.isClickAuto = false;
	self:setAutoShow();
end

function Lua_TownTreasureWindow:resetCheckSelect()
	DataCenter.townTreasureData.isOpenAuto = false;
	self.checkBox.selected = false;
	self.isClickAuto = false;
	self:setAutoShow();
end